package lu;

import java.util.*;

import edu.isi.gamebots.client.*;
import edu.isi.gamebots.examples.*;
import lu.*;

public  class UTShop {
  private static UTShop  thisObject;
  private HashMap botMap= new HashMap();
  private HashMap domList = new HashMap();


 //This construction is hidden from the user, can only access object from getObject()
  private UTShop() {
  }

  /**
   * This functions returns this object. If null, it creates one before returning it
   * This is synchronized to prevent a race condition where 2 object could be created
   * @return UTShop
   */
  synchronized public static UTShop getObject(){
    if (thisObject == null){
      thisObject = new UTShop();
    }
    return thisObject;
  }


  synchronized public void addBot(String botName) {

    botMap.put(botName,new UTShopBot(botName));
  }

 synchronized public void removeBot (String botName) {
    botMap.remove(botName);
  }

  synchronized public void updateSelfInfo(HTNbot bot){
    String team;
    if (bot == null) return;
    String name = bot.getName();

    if (name == null) return;

    UTShopBot temp = (UTShopBot) botMap.get(name);
    if (temp != null) {
      //name
      temp.setBotName( name );
      //team
      temp.setBotTeam(  Integer.toString(bot.team) );
      //loc
      temp.setBotLoc( new Vector3D( bot.x, bot.y, bot.z));
    }

  }

  synchronized public void setLocToGoTo(String botName, Vector3D loc){
    if (botName == null || loc == null) return;
    UTShopBot temp = (UTShopBot) botMap.get(botName);

    if (temp != null) {
      temp.setCmd(loc);
     // botMap.put(botName, temp);  //this needed to be fixed
    }

  }

  synchronized public void setLocToGoTo(String botName, String loc){
    if (botName == null || loc == null) return;
    UTShopBot temp = (UTShopBot) botMap.get(botName);
    double[] vector = null;
    if (temp !=null ){
      try {
         vector = edu.isi.gamebots.client.Bot.parseVector(loc);
      }
      catch (NumberFormatException e){
        return;
      }

      Vector3D temploc = new Vector3D(vector[0], vector[1], vector[2]);

      temp.setCmd(temploc);
    }

  }



  synchronized public Vector3D getLocToGoTo(String botName){
    Vector3D retLoc= null;
    if (botName == null) return null;
    UTShopBot temp = (UTShopBot) botMap.get(botName);

    if (temp != null) {
      retLoc= temp.getCmd() ;  //return the loc
      temp.clearCmd() ;        //clear the loc
    }
    return retLoc ;
  }

  synchronized public void  atLocToGoTo(String botName){
    if (botName == null) return ;
    UTShopBot temp = (UTShopBot) botMap.get(botName);
    if (temp != null) {
      temp.setAssigned(false);   //hah3
    }

  }


  public int getBotCount(){
    return botMap.size();
  }


  synchronized public String getBotAndAssign(String botName ,String teamNo){
    UTShopBot temp;
    int whichTeam;
    int botTeam ;


    try {
      whichTeam = (int) Double.parseDouble(teamNo);
    }
    catch (NumberFormatException e) {
      whichTeam = 257;
    }



    Iterator it = botMap.values().iterator() ;
    while (it.hasNext() ){
      temp = (UTShopBot) it.next();


      try {
        botTeam = (int) Integer.parseInt(temp.getBotTeam());
      }
      catch (NumberFormatException e) {
        botTeam = 256;
      }




      if (!temp.isAssigned() &&  temp.getBotName()!=null && temp.getBotName().equalsIgnoreCase(botName) && whichTeam == botTeam ) {
        temp.setAssigned(true);
        return temp.getBotName();
      }

    }

    return null;
  }

  synchronized public Vector3D getBotLoc(String botName ){
    UTShopBot temp;
    if (botName == null) return null;

    Iterator it = botMap.values().iterator() ;
    while (it.hasNext() ){
      temp = (UTShopBot) it.next();

      if ( temp != null && temp.getBotName()!=null && temp.getBotName().equalsIgnoreCase(botName) ) {
        return temp.getBotLoc() ;
      }

    }
    return null;
  }


  synchronized public Vector getListOfBot( ){
    Vector retList=new Vector();

    if (botMap.size() ==0) return null;

    Iterator it = botMap.keySet().iterator() ;
    while (it.hasNext() ){
      retList.add( it.next() );
    }

    return retList;
  }

  synchronized public String getBotTeam(String botName){
    UTShopBot temp;
    Iterator it = botMap.values().iterator() ;

    while (it.hasNext() ){
      temp = (UTShopBot) it.next();
      if ( temp != null && temp.getBotName()!=null && temp.getBotName().equalsIgnoreCase(botName)  ) {
        return temp.getBotTeam();
      }
    }
    return null;
  }

  synchronized public boolean isBotNotAssigned(String botName){
    UTShopBot temp;
    Iterator it = botMap.values().iterator() ;

    while (it.hasNext() ){
      temp = (UTShopBot) it.next();
      if (temp != null && !temp.isAssigned() && temp.getBotName()!=null && temp.getBotName().equalsIgnoreCase(botName) ) {
        return true;
      }
    }
    return false;
  }

  synchronized public void addDomPoint( UTPNode dom_point){
     domList.put(dom_point.getID(),dom_point);
   }

//updated 4/3/05
   synchronized public void updateDomPointOwner( String domId, int domOwner){
     UTPNode node=null;
     Iterator iter = domList.values().iterator();
     while (iter.hasNext()) {
       node = (UTPNode) iter.next();
       if (node != null && node.getID().equalsIgnoreCase(domId) ) {
        // System.out.println("node id updatign " + node.getID() + "   "+node.getOwner());
         node.setOwner(domOwner);
        // System.out.println("node id updateddddddddddd " + node.getID() + "  " + node.getOwner());
       }
     }

   }

  synchronized public Vector getDomList(){
    Vector retVal = new Vector();
    Iterator iter = domList.values().iterator();
    while (iter.hasNext()){
      retVal.add( iter.next() );
    }
    return  retVal;
  }
}
